@@ -38,7 +38,7 @@
 #include "xstuff.h"
 
 #define ICON_SPACING         1       /* spacing between icons */
-#define ICON_SIZE_PANEL      24      /* icons size on panels */
+#define ICON_SIZE_PANEL      22      /* icons size on panels */
 #define ICON_SIZE_MENU       22      /* icons size in menus */
 #define DEFAULT_MIN_VISIBLE_COLS 2   /* minimum number of columns to display */
 #define DEFAULT_MAX_VISIBLE_COLS 20  /* maximum number of columns to display */
@@ -125,14 +125,15 @@ get_arrow_widget (QuickBox *quick_box)
 static int
 count_columns (QuickBox *quick_box)
 {
-	GList *children = GTK_BOX (quick_box)->children;
+	GList *children, *scan;
 	int    col = 0;
 	int    row = 0;
 
-	for (; children; children = children->next) {
-		GtkBoxChild *child = children->data;
+	children = gtk_container_get_children (GTK_CONTAINER (quick_box));
+	for (scan = children; scan; scan = scan->next) {
+		GtkWidget *child_widget = scan->data;
 
-		if (QUICK_IS_SEPARATOR (child->widget)) {
+		if (QUICK_IS_SEPARATOR (child_widget)) {
 			if (row > 0)
 				col++;
 			col++;
@@ -145,6 +146,7 @@ count_columns (QuickBox *quick_box)
 			}
 		}			
 	}
+	g_list_free (children);
 
 	if (row > 0)
 		col++;
@@ -164,7 +166,7 @@ quick_box_size_request (GtkWidget        *widget,
 	int              arrow_width, arrow_height;
 	GtkOrientation   orientation;
 	GtkRequisition   child_requisition;
-	GList           *children;
+	GList           *children, *scan;
 	int              spacing;
 	int              min_columns, max_columns;
 	int              row, col;
@@ -201,15 +203,16 @@ quick_box_size_request (GtkWidget        *widget,
 	col = 1;
 	row = 1;
 
-	for (children = box->children; children; children = children->next) {
-		GtkBoxChild    *child = children->data;
+	children = gtk_container_get_children (GTK_CONTAINER (box));
+	for (scan = children; scan; scan = scan->next) {
+		GtkWidget      *child_widget = scan->data;
 		GtkRequisition  child_requisition;
 
-		gtk_widget_size_request (GTK_WIDGET (child->widget), 
+		gtk_widget_size_request (GTK_WIDGET (child_widget), 
 					 &child_requisition);
 
 		if ((orientation == GTK_ORIENTATION_HORIZONTAL)
-		    && QUICK_IS_SEPARATOR (child->widget)) {
+		    && QUICK_IS_SEPARATOR (child_widget)) {
 
 			if (max_column_width != 0) {
 				p->col_size[col - 1] = max_column_width + spacing;
@@ -236,7 +239,7 @@ quick_box_size_request (GtkWidget        *widget,
 			row = 1;
 
 		} else if ((orientation == GTK_ORIENTATION_HORIZONTAL)
-			   && QUICK_IS_BUTTON (child->widget)) {
+			   && QUICK_IS_BUTTON (child_widget)) {
 
 			max_column_width = MAX (max_column_width, 
 						child_requisition.width);
@@ -255,7 +258,7 @@ quick_box_size_request (GtkWidget        *widget,
 			} 
 			
 		} else if ((orientation == GTK_ORIENTATION_VERTICAL)
-			   && QUICK_IS_SEPARATOR (child->widget)) {
+			   && QUICK_IS_SEPARATOR (child_widget)) {
 			
 			if (max_column_height != 0) {
 				p->col_size[row - 1] = max_column_height + spacing;
@@ -282,7 +285,7 @@ quick_box_size_request (GtkWidget        *widget,
 			col = 1;
 
 		} else if ((orientation == GTK_ORIENTATION_VERTICAL)
-			   && QUICK_IS_BUTTON (child->widget)) {
+			   && QUICK_IS_BUTTON (child_widget)) {
 			
 			max_column_height = MAX (max_column_height, 
 						 child_requisition.height);
@@ -339,17 +342,27 @@ quick_box_size_request (GtkWidget        *widget,
 	arrow_height = child_requisition.height;
 
 	if (orientation == GTK_ORIENTATION_HORIZONTAL) { 
-		min_size += spacing + arrow_width;
+		int last_col = col;
+		if (row == 1)
+			last_col--;
+		if (last_col != min_columns)
+			min_size += spacing + arrow_width;
+
 		max_size += spacing + arrow_width;
-		if (box->children == NULL) /* when the box is empty */
+		if (children == NULL) /* when the box is empty */
 			min_size = p->size / p->rows; 
 		requisition->width = min_size;
 		requisition->height = MAX (p->size, arrow_height);
 		p->size_hint[SIZE_HINT_MAX] = max_size - requisition->width;
 	} else {
-		min_size += spacing + arrow_height;
+		int last_row = row;
+		if (col == 1)
+			last_row--;
+		if (last_row != min_columns)
+			min_size += spacing + arrow_height;
+
 		max_size += spacing + arrow_height;
-		if (box->children == NULL) /* when the box is empty */
+		if (children == NULL) /* when the box is empty */
 			min_size = p->size / p->rows; 
 		requisition->width = MAX (p->size, arrow_width);
 		requisition->height = min_size;
@@ -358,14 +371,16 @@ quick_box_size_request (GtkWidget        *widget,
 
 	p->size_hint[SIZE_HINT_MIN] = 0;
 
-	/* FIXME
-	  if (! GTK_WIDGET_VISIBLE (arrow)) {
-	  if (orientation == GTK_ORIENTATION_HORIZONTAL)
-	  p->size_hint[SIZE_HINT_MAX] -= spacing + arrow_width;
-	  else
-	  p->size_hint[SIZE_HINT_MAX] -= spacing + arrow_height;
-	  }
-	*/
+	/* FIXME */
+	if (! GTK_WIDGET_VISIBLE (arrow)) {
+		if (orientation == GTK_ORIENTATION_HORIZONTAL)
+			p->size_hint[SIZE_HINT_MAX] -= spacing + arrow_width;
+		else
+			p->size_hint[SIZE_HINT_MAX] -= spacing + arrow_height;
+	}
+	/**/
+
+	g_list_free (children);
 	
 	/* add border width */
 
@@ -396,12 +411,8 @@ quick_box_forall (GtkContainer *container,
 		  GtkCallback   callback,
 		  gpointer      callback_data)
 {
-	GtkBox *box;
-	
 	g_return_if_fail (callback != NULL);
 	
-	box = GTK_BOX (container);
-
 	GTK_CONTAINER_CLASS (parent_class)->forall (container,
 						    include_internals,
 						    callback,
@@ -414,6 +425,26 @@ quick_box_forall (GtkContainer *container,
 }
 
 
+#if 0
+static void
+quick_box_add (GtkContainer *container,
+	       GtkWidget    *widget)
+{
+	g_print ("ADD\n");
+	GTK_CONTAINER_CLASS (parent_class)->add (container, widget);
+}
+
+
+static void 
+quick_box_remove (GtkContainer *container,
+		  GtkWidget    *widget)
+{
+	g_print ("REMOVE\n");
+	GTK_CONTAINER_CLASS (parent_class)->remove (container, widget);
+}
+#endif
+
+
 static void
 hide_non_current_arrow_cb (GtkWidget *arrow, 
 			   QuickBox  *quick_box)
@@ -456,18 +487,19 @@ children_under_the_arrow (QuickBox *quick_box,
 
 
 static gboolean
-last_n_child (QuickBox *quick_box, 
-	      GList    *children, 
-	      int       last_buttons)
+last_n_child (QuickBox  *quick_box, 
+	      GtkWidget *child_widget,
+	      int        last_buttons)
 {
-	GList *scan;
+	GList *children, *scan;
 
-	scan = g_list_last (GTK_BOX (quick_box)->children);
-	for (; (last_buttons > 0) && scan; scan = scan->prev) {
-		if (scan->data == children->data)
+	children = gtk_container_get_children (GTK_CONTAINER (quick_box));
+	for (scan = g_list_last (children); (last_buttons > 0) && scan; scan = scan->prev) {
+		if (scan->data == child_widget)
 			return TRUE;
 		last_buttons--;
 	}
+	g_list_free (children);
 
 	return FALSE;
 }
@@ -477,14 +509,15 @@ static void
 show_first_n_children (QuickBox *quick_box,
 		       int       n)
 {
-	GList *children = GTK_BOX (quick_box)->children;
-	GList *scan;
+	GList *children, *scan;
 
+	children = gtk_container_get_children (GTK_CONTAINER (quick_box));
 	for (scan = children; (n > 0) && scan; scan = scan->next) {
-		GtkBoxChild *child = scan->data;
-		gtk_widget_show (child->widget);
+		GtkWidget *child_widget = scan->data;
+		gtk_widget_show (child_widget);
 		n--;
 	}
+	g_list_free (children);
 }
 
 
@@ -492,14 +525,15 @@ static void
 hide_last_n_children (QuickBox *quick_box,
 		      int       n)
 {
-	GList *children = GTK_BOX (quick_box)->children;
-	GList *scan;
+	GList *children, *scan;
 
+	children = gtk_container_get_children (GTK_CONTAINER (quick_box));
 	for (scan = g_list_last (children); (n > 0) && scan; scan = scan->prev) {
-		GtkBoxChild *child = scan->data;
-		gtk_widget_hide (child->widget);
+		GtkWidget *child_widget = scan->data;
+		gtk_widget_hide (child_widget);
 		n--;
 	}
+	g_list_free (children);
 }
 
 
@@ -538,7 +572,7 @@ quick_box_size_allocate (GtkWidget        *widget,
 	int             initial_x, initial_y;
 	GtkRequisition  child_requisition;
 	gboolean        ignoring_arrow, do_not_fit = FALSE;
-	GList          *children, *hide_from_here = NULL;
+	GList          *children, *scan, *hide_from_here = NULL;
 	int             under_arrow_children;
 	int             children_to_hide, n_children;
 	gboolean        second_try = FALSE;
@@ -595,12 +629,13 @@ quick_box_size_allocate (GtkWidget        *widget,
 		under_arrow_children = children_under_the_arrow (quick_box, arrow_height);
 	ignoring_arrow = TRUE;
 
-	for (children = box->children; children; ) {
-		GtkBoxChild   *child = children->data;
+	children = gtk_container_get_children (GTK_CONTAINER (quick_box));
+	for (scan = children; scan; ) {
+		GtkWidget     *child_widget = scan->data;
 		GtkAllocation  child_allocation;
 
 		if (ignoring_arrow 
-		    && last_n_child (quick_box, children, under_arrow_children)) {
+		    && last_n_child (quick_box, child_widget, under_arrow_children)) {
 			ignoring_arrow = FALSE;
 			if (orientation == GTK_ORIENTATION_HORIZONTAL) 
 				max_width += arrow_width + spacing;
@@ -610,7 +645,7 @@ quick_box_size_allocate (GtkWidget        *widget,
 
 		child_allocation.x = x;
 		child_allocation.y = y;
-		gtk_widget_get_child_requisition (child->widget, &child_requisition);
+		gtk_widget_get_child_requisition (child_widget, &child_requisition);
 		child_allocation.width = child_requisition.width;
 		child_allocation.height = child_requisition.height;
 
@@ -631,12 +666,12 @@ quick_box_size_allocate (GtkWidget        *widget,
 			if (x + child_allocation.width > max_width) {
 				if (! do_not_fit) {
 					do_not_fit = TRUE;
-					hide_from_here = children;
+					hide_from_here = scan;
 				}
 			} else if (current_height > max_height) { /* new row */
 				if (second_try) {
 					do_not_fit = TRUE;
-					hide_from_here = children;
+					hide_from_here = scan;
 				} else {
 					x += max_line_width + spacing;
 					y = initial_y;
@@ -658,12 +693,12 @@ quick_box_size_allocate (GtkWidget        *widget,
 			if (y + child_allocation.height > max_height) {
 				if (! do_not_fit) {
 					do_not_fit = TRUE;
-					hide_from_here = children;
+					hide_from_here = scan;
 				}
 			} else if (current_width > max_width) { /* new row */
 				if (second_try) {
 					do_not_fit = TRUE;
-					hide_from_here = children;
+					hide_from_here = scan;
 				} else {
 					x = initial_x;
 					y += max_line_height + spacing;
@@ -675,7 +710,7 @@ quick_box_size_allocate (GtkWidget        *widget,
 			} 
 		}
 
-		gtk_widget_size_allocate (child->widget, &child_allocation);
+		gtk_widget_size_allocate (child_widget, &child_allocation);
 
 		if (orientation == GTK_ORIENTATION_HORIZONTAL) {
 			max_line_width = MAX (max_line_width, child_allocation.width);
@@ -688,7 +723,7 @@ quick_box_size_allocate (GtkWidget        *widget,
 		}
 
 		second_try = FALSE;
-		children = children->next;
+		scan = scan->next;
 	}
 
 	if (do_not_fit) {
@@ -724,7 +759,7 @@ quick_box_size_allocate (GtkWidget        *widget,
 	} else 
 		children_to_hide = 0;
 
-	n_children = g_list_length (box->children);
+	n_children = g_list_length (children);
 	show_first_n_children (quick_box, n_children - children_to_hide);
 
 	/**/
@@ -749,6 +784,8 @@ quick_box_size_allocate (GtkWidget        *widget,
 	else
 		widget->allocation.height = MAX (widget->allocation.height, 
 						 child_requisition.height);
+
+	g_list_free (children);
 }
 
 
@@ -759,7 +796,7 @@ get_drag_destination_rect (QuickBox     *quick_box,
 {
 	GtkWidget *widget = GTK_WIDGET (quick_box);
 	GtkBox    *box = GTK_BOX (quick_box);
-	GList     *scan;
+	GList     *children, *scan;
 	int        i, cx, cy, cwidth, cheight;
 
 	cx = widget->allocation.x;
@@ -767,18 +804,20 @@ get_drag_destination_rect (QuickBox     *quick_box,
 	cwidth = widget->allocation.width;
 	cheight = widget->allocation.height;
 	
+	children = gtk_container_get_children (GTK_CONTAINER (quick_box));
 	i = 0;
-	for (scan = box->children; scan && (i <= pos); scan = scan->next) {
-		GtkBoxChild *child = scan->data;
+	for (scan = children; scan && (i <= pos); scan = scan->next) {
+		GtkWidget *child_widget = scan->data;
 
-		cx = child->widget->allocation.x;
-		cy = child->widget->allocation.y;
-		cwidth = child->widget->allocation.width;
-		cheight = child->widget->allocation.height;
+		cx = child_widget->allocation.x;
+		cy = child_widget->allocation.y;
+		cwidth = child_widget->allocation.width;
+		cheight = child_widget->allocation.height;
 		
 		i++;
 	}
-	
+	g_list_free (children);
+
 	rect->x = cx;
 	rect->y = cy;
 	rect->width = cwidth;
@@ -830,7 +869,7 @@ quick_box_expose (GtkWidget        *widget,
 	}
 
 	if (GTK_WIDGET_HAS_FOCUS (widget) 
-	    && g_list_length (GTK_BOX (widget)->children) == 0) {
+	    && (_gtk_container_get_n_children (GTK_CONTAINER (widget)) == 0)) {
 		int focus_width, focus_pad;
 		int x, y, width, height;
 		
@@ -901,6 +940,11 @@ quick_box_class_init (QuickBoxClass    *class)
 	widget_class->size_allocate = quick_box_size_allocate;
 	widget_class->button_press_event = quick_box_button_press;
 	widget_class->expose_event = quick_box_expose;
+
+#if 0
+	container_class->add = quick_box_add;
+	container_class->remove = quick_box_remove;
+#endif
 	container_class->forall = quick_box_forall;
 }
 
@@ -1010,7 +1054,7 @@ button_button_press_cb (GtkWidget      *button,
 
 	popup_menu = gtk_menu_new ();
 	gtk_menu_set_screen (GTK_MENU (popup_menu),
-			     gtk_widget_get_screen (button));
+			     gtk_widget_get_screen (GTK_WIDGET (quick_box)));
 
 	if (quick_box->priv->popup_menu != NULL)
 		g_signal_connect_swapped (G_OBJECT (popup_menu),
@@ -1079,7 +1123,7 @@ menu_item__drag_data_get  (GtkWidget        *widget,
         gtk_selection_data_set (selection_data, 
                                 selection_data->target,
                                 8, 
-				uri,
+				(guchar*) uri,
                                 strlen (uri));
 }
 
@@ -1087,43 +1131,35 @@ menu_item__drag_data_get  (GtkWidget        *widget,
 static void deactivate_arrow (QuickBox *quick_box);
 
 
-static int
-arrow_button_pressed (GtkToggleButton *tbutton,
-		      QuickBox        *quick_box)
+static void
+display_popup_menu (QuickBox *quick_box)
 {
-	GtkWidget  *popup_menu = NULL;
-	GtkWidget  *item;
-	GList      *children;
-	gboolean    menu_empty = TRUE;
-
-	gtk_tooltips_set_tip (quick_box->tooltips, 
-			      GTK_WIDGET (tbutton),
-			      _("Hide launchers menu"),
-			      NULL);
+	GtkWidget *popup_menu = NULL;
+	GtkWidget *item;
+	GList     *children = NULL, *scan;
+	gboolean   menu_empty = TRUE;
 
 	if (quick_box->priv->popup_menu != NULL) {
 		gtk_menu_shell_deactivate (GTK_MENU_SHELL (quick_box->priv->popup_menu));
-		return TRUE;
+		quick_box->priv->popup_menu = NULL;
+		return;
 	}
 
-	for (children = GTK_BOX (quick_box)->children; children; children = children->next) {
+	children = gtk_container_get_children (GTK_CONTAINER (quick_box));
+
+	for (scan = children; scan; scan = scan->next) {
+		GtkWidget        *child_widget = scan->data;
 		GnomeDesktopItem *ditem;
-		GtkBoxChild      *child;
 		GtkWidget        *icon = NULL;
 		char             *icon_path = NULL;
 		const char       *text;
 		const char       *comment;
 		char             *tip;
 
-		child = children->data;
-
-		if ((child == NULL)
-		    || (child->widget == NULL)
-		    || ! GTK_IS_WIDGET (child->widget)
-		    || GTK_WIDGET_VISIBLE (child->widget))
+		if ((child_widget != NULL) && GTK_WIDGET_VISIBLE (child_widget))
 			continue;
-		
-		ditem = g_object_get_data (G_OBJECT (child->widget), "ditem");
+
+		ditem = g_object_get_data (G_OBJECT (child_widget), "ditem");
 
 		if (ditem == NULL) { /* add separator */
 			
@@ -1164,7 +1200,10 @@ arrow_button_pressed (GtkToggleButton *tbutton,
 		/**/
 
 		comment = gnome_desktop_item_get_localestring (ditem, "Comment"); 
-		tip = g_strconcat (((comment != NULL) && (*comment != 0)) ? comment : text, NULL);
+		if ((comment != NULL) && (*comment != 0))
+			tip = g_strdup (comment);
+		else
+			tip = g_strdup (text);
 		gtk_tooltips_set_tip (quick_box->tooltips, 
 				      GTK_WIDGET (item),
 				      tip,
@@ -1181,11 +1220,11 @@ arrow_button_pressed (GtkToggleButton *tbutton,
 		g_signal_connect_swapped (G_OBJECT (item), 
 					  "activate",
 					  G_CALLBACK (item_activated), 
-					  child->widget);
+					  child_widget);
 		g_signal_connect_swapped (G_OBJECT (item), 
 					  "button_press_event",
 					  G_CALLBACK (button_button_press_cb), 
-					  child->widget);		
+					  child_widget);		
 
 		/**/
 
@@ -1197,7 +1236,7 @@ arrow_button_pressed (GtkToggleButton *tbutton,
 		g_signal_connect (G_OBJECT (item),
 				  "drag_data_get",
 				  G_CALLBACK (menu_item__drag_data_get), 
-				  child->widget);
+				  child_widget);
 
 		g_signal_connect (G_OBJECT (item),
 				  "drag_begin",
@@ -1214,11 +1253,18 @@ arrow_button_pressed (GtkToggleButton *tbutton,
 		gtk_widget_show (item);
 	}
 
+	g_list_free (children);
+
 	if (menu_empty) {
 		deactivate_arrow (quick_box);
-		return TRUE;
+		return;
 	}
 
+	gtk_tooltips_set_tip (quick_box->tooltips, 
+			      get_arrow_widget (quick_box),
+			      _("Hide launchers menu"),
+			      NULL);
+
 	g_signal_connect_swapped (G_OBJECT (popup_menu),
 				  "deactivate",
 				  G_CALLBACK (deactivate_arrow), 
@@ -1230,7 +1276,15 @@ arrow_button_pressed (GtkToggleButton *tbutton,
 			quick_box, 
 			0,
 			GDK_CURRENT_TIME);
+}
 
+
+static int
+arrow_button_toggled (GtkToggleButton *tbutton,
+		      QuickBox        *quick_box)
+{
+	g_return_val_if_fail (QUICK_IS_BOX (quick_box), TRUE);
+	display_popup_menu (quick_box);
 	return TRUE;
 }
 
@@ -1242,7 +1296,7 @@ deactivate_arrow (QuickBox *quick_box)
 
 	arrow = get_arrow_widget (quick_box);
 	g_signal_handlers_block_by_func (G_OBJECT (arrow),
-					 G_CALLBACK (arrow_button_pressed),
+					 G_CALLBACK (arrow_button_toggled),
 					 quick_box);
 	
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (arrow), FALSE);
@@ -1252,7 +1306,7 @@ deactivate_arrow (QuickBox *quick_box)
 			      NULL);
 	
 	g_signal_handlers_unblock_by_func (G_OBJECT (arrow),
-					   G_CALLBACK (arrow_button_pressed),
+					   G_CALLBACK (arrow_button_toggled),
 					   quick_box);
 
 	quick_box->priv->popup_menu = NULL;
@@ -1281,7 +1335,7 @@ create_arrow_button (QuickBox     *quick_box,
 
 	g_signal_connect (G_OBJECT (button), 
 			  "toggled",
-			  G_CALLBACK (arrow_button_pressed), 
+			  G_CALLBACK (arrow_button_toggled), 
 			  quick_box);
 	gtk_widget_hide (button);
 
@@ -1297,6 +1351,8 @@ quick_box_init (QuickBox *quick_box)
 
 	box->spacing     = ICON_SPACING;
 	box->homogeneous = FALSE;
+	gtk_container_set_resize_mode (GTK_CONTAINER (quick_box),
+				       GTK_RESIZE_QUEUE);
 
 	priv = quick_box->priv = g_new0 (QuickBoxPrivate, 1);
 	priv->size                    = ICON_SIZE_PANEL;
@@ -1359,17 +1415,18 @@ quick_box_get_pointer_position (QuickBox *quick_box,
 				int       y)
 {
 	GtkBox *box = GTK_BOX (quick_box);
-	GList  *scan;
+	GList  *children, *scan;
 	int     pos = 0;
 	int     same_col_pos = -1;
 	int     same_row_pos = -1;
 
-	if (box->children == NULL)
+	children = gtk_container_get_children (GTK_CONTAINER (quick_box));
+	if (children == NULL)
 		return -1;
 
-	for (scan = box->children; scan; scan = scan->next) {
-		GtkBoxChild   *child = scan->data;
-		GdkRectangle   rect = child->widget->allocation;
+	for (scan = children; scan; scan = scan->next) {
+		GtkWidget    *child_widget = scan->data;
+		GdkRectangle  rect = child_widget->allocation;
 		
 		rect.x      -= box->spacing;
 		rect.y      -= box->spacing;
@@ -1388,6 +1445,7 @@ quick_box_get_pointer_position (QuickBox *quick_box,
 
 		pos++;
 	}
+	g_list_free (children);
 
 	if (get_gtk_orientation (quick_box) == GTK_ORIENTATION_HORIZONTAL)
 		return same_col_pos + 1;
@@ -1439,7 +1497,7 @@ drag_motion (GtkWidget          *widget,
 	int       pos;
 
 	pos = quick_box_get_pointer_position (quick_box, x, y);
-	if ((pos == -1) && (GTK_BOX (widget)->children == NULL))
+	if ((pos == -1) && (_gtk_container_get_n_children (GTK_CONTAINER (widget)) == 0))
 		pos = 0;
 	draw_drag_destination (quick_box, pos);
 
@@ -1478,6 +1536,34 @@ button_drag_started_cb (GtkWidget *button,
 }
 
 
+static void
+icon_theme_changed (GnomeIconTheme *icon_theme,
+		    QuickBox       *quick_box)
+{
+	GList *children, *scan;
+	int    child_size = get_child_size (quick_box);
+
+	children = gtk_container_get_children (GTK_CONTAINER (quick_box));
+	for (scan = children; scan; scan = scan->next) {
+		GtkWidget        *child_widget = scan->data;
+		GnomeDesktopItem *ditem;
+		char             *icon_path;
+
+		if (! QUICK_IS_BUTTON (child_widget))
+			continue;
+
+		ditem = g_object_get_data (G_OBJECT (child_widget), "ditem");
+		if (ditem == NULL)
+			continue;
+
+		icon_path = gnome_desktop_item_get_icon (ditem, quick_box->priv->icon_theme);
+		quick_button_set_pixmap (QUICK_BUTTON (child_widget), icon_path, child_size);
+		g_free (icon_path);
+	}
+	g_list_free (children);
+}
+
+
 GtkWidget *
 quick_box_add_button (QuickBox     *quick_box,
 		      const char   *uri,
@@ -1606,6 +1692,10 @@ quick_box_new (PanelAppletOrient  orient,
 			  "drag_leave",
 			  G_CALLBACK (drag_leave), 
 			  quick_box);
+	g_signal_connect (G_OBJECT (quick_box->priv->icon_theme),
+			  "changed",
+			  G_CALLBACK (icon_theme_changed),
+			  quick_box);
 
 	return GTK_WIDGET (quick_box);
 }
